pro examplerun

;; START HERE: To use this integrator, you do not need to look at any
;; files in the directories idl_utilities or mercury_files.
;; The only files you need to modify are get_intial_conditions.pro
;; (as noted below) and this file (examplerun.pro).  Start by copying
;; this file and giving it a new name.  Modify the top to change the
;; parameters for your run and modify the bottom to generate plots
;; of your output.

;;;; IMPORTANT NOTE: Add the following line to your IDL_STARTUP file
;;;; and then comment it out below to avoid adding the idl_utilities
;;;; direcroty to your path multiple times.

!path = 'idl_utilities:' + !path

;;;;;  TO SET UP YOUR OWN INTEGRATION  ;;;;;
; Set the properties of the planets in your system
; in the file get_initial_conditions.pro
;
; Then set the parameters below.

mcen = 1.0 ; central mass in solar masses

algorithm = 'bs' ; bs = Bulirsch-Stoer; hyb = hybrid

duration = 200.0 ; duration of integration in years
outstep = 0.1 ; time between outputs in years (must be > 0.022 for default settings)


;;;; LEAVE THIS SECTION ALONE ;;;;
;;;; GO TO BOTTOM TO ADJUST OUTPUT ;;;;;

dir='mercury_files'

stopt = duration*365.25 ; convert to days
output = outstep*365.25 ; convert to days
print_param_in, dir=dir, stopt=stopt, output=output, mcen=mcen, algorithm=algorithm 

; Input the properties of the planets
get_initial_conditions, orbstart
nbodies = n_elements(orbstart)

;; Set the properties of the massive bodies for the integration
print_big_in, orbstates=orbstart, /double, dir=dir

;; Perform the integration
integrate_merc, dir=dir

;; Convert the output files into IDL sav files
for i=0,nbodies-1 do begin
  aeitosav, objname=orbstart[i].name, dir=dir
endfor

;; Read in the results
restore, dir + '/' + orbstart[0].name+'.sav'  ; read in the first one to get the number of times
nts = n_elements(orb) ; the number of times
orb1 = {rmc_orbstruct}
orbs = replicate(orb1, nbodies, nts) ; the structure to put the results in
orbs[0,*] = orb
; now read in the rest
for i=1,nbodies-1 do begin
  restore, dir + '/' + orbstart[i].name+'.sav'
  orbs[i,*] = orb
endfor
;;;;; END OF SECTION TO LEAVE ALONE ;;;;;;


;;;;;; OUTPUT: PLOTTING and SAVING ;;;;;;

;;;;; Save any good results to a different directory ;;;;;
;;; Note that this does NOT save any plots that you have made.
keepoutput_merc, 'example_saved_output', startdir=dir


;; You now have an array of structures that hold the properties
;; of each object's orbit, including its osculating elements,
;; position, and velocity as a function of time.
;; Our default setup generates orbital elements relative to the
;; central star (as opposed to the system's center of mass).
;; The array 'orbs' has two indices: 1) the index of the object,
;; 2) the time.  For example, orbs[0,8] gives the properties of object
;; 0 (in our example, Jupiter) at output timestep 8 (note that this is
;; actually the 9th output time since array indices start at 0).
;; To get a particular piece of information about the orbit, add a '.'
;; and the name of that piece of information.  For example,
;; orbs[0,8].a is the semi-major axis (in AU) of Jupiter at timestep
;; 8, orbs[0,8].time is the time at timestep 8, and
;; orbs[0,8].name is 'Jupiter' (Jupiter's name is Jupiter at every
;; timestep).  

;; Here is a list of fields in the structure and what they contain:
;; All values are doubles except name, which is a character string.

;;          name:     ;; name of object
;;          mass:     ;; mass of object (solar masses)
;;          time:     ;; time at which these values are valid (yr)
;;          a:        ;; semi-major axis (AU)
;;          e:        ;; eccentricity
;;          i:        ;; inclination (radians)
;;          peri:     ;; argument of pericenter (radians)
;;          node:     ;; longitude of ascending node (radians)
;;          M:        ;; mean anomaly (radians)
;;          r:        ;; distance from the central body or center of mass (AU)
;;          f:        ;; true anomaly (radians)
;;          lambda:   ;; mean longitude (radians)
;;          pomega:   ;; longitude of pericenter (radians)
;;          x:        ;; cartesian position in x direction (AU)
;;          y:        ;; cartesian position in y direction (AU)
;;          z:        ;; cartesian position in z direction (AU)
;;          vx:       ;; cartesian velocity in x direction (AU/yr)
;;          vy:       ;; cartesian velocity in y direction (AU/yr)
;;          vz:       ;; cartesian velocity in z direction (AU/yr)


;;;;;; Plot the results to the screen ;;;;;;
setcolors, /system_variables  ; include this line if you want to use color
; setcolors, /test gives you a list of predefined colors

plot, [0], [0], /nodata, xrange=[0,200], yrange=[-4,4], /xstyle, /ystyle, xtitle='Time (yr)', ytitle='x Velocity (AU/yr)'
oplot, orbs[0,*].time, orbs[0,*].vx   ; x velocities  in AU/yr  ; Jupiter
oplot, orbs[1,*].time, orbs[1,*].vx, linestyle=2  ; Saturn
oplot, orbs[2,*].time, orbs[2,*].vx, color=!red  ; Uranus
oplot, orbs[3,*].time, orbs[3,*].vx, color=!red, linestyle=2  ; Neptune


;;;;;;; Now output the plot to a postscript file ;;;;;;
psopen, 'exampleplot.eps', /encapsulated, /portrait, /times, xsize=30, ysize=20, /color ;  open an encapsulated postscript file with Times font
setcolors, /system_variables

; make your plot; font=0 means use postscript font
; IDL plots need large values (e.g. 7) for xthick, ythick, and thick to make
; the lines thick enough.  If you want legible axes, pick a large
; enough charsize.
plot, [0], [0], /nodata, xrange=[0,200], yrange=[-4,4], /xstyle, /ystyle, font=0, charsize=3, xthick=7, ythick=7, xtitle='Time (yr)', ytitle='x Velocity (AU/yr)'

oplot, orbs[0,*].time, orbs[0,*].vx, thick=5
oplot, orbs[1,*].time, orbs[1,*].vx, thick=5, linestyle=2
oplot, orbs[2,*].time, orbs[2,*].vx, thick=5, color=!red
oplot, orbs[3,*].time, orbs[3,*].vx, thick=5, linestyle=2, color=!red

psclose ; close the postscript file


stop
end
